import RegFile::*;
import RWire::*;
import ProcTypes::*;

`include "asim/provides/low_level_platform_interface.bsh"
`include "asim/provides/soft_connections.bsh"
`include "asim/provides/fpga_components.bsh"
`include "asim/provides/common_services.bsh"

//-----------------------------------------------------------
// Register file module
//-----------------------------------------------------------

interface BRFile;
   method Action   wr( Rindx rindx, Bit#(32) data );
   method Action Value Bit#(32) rd1( Rindx rindx );
   method Action Value Bit#(32) rd2( Rindx rindx );
endinterface

(* doc = "synthesis attribute ram_style mkBRegFile distributed;" *)
(* synthesize *)
module mkBRegFile(BRFile) 
   provisos (Bits#(index_t, size_index),
	     Bits#(data_t, size_data),
	     Eq#(index_t),
	     Bounded#(index_t) );

   LUTRAM#(index_t, data_t) rf <- mkLUTRAMU_RegFile();
   RWire#(Tuple2#(index_t, data_t)) rw <-mkRWire();

   method Action wr( Rindx rindx, Bit#(32) data );
      rf.upd( rindx, data );
      rw.wset(tuple2(rindex,data));
   endmethod

   method Bit#(32) rd1 (Rindx r);
      if (r == 0) return 0;
      else begin
            case (rw.wget()) matches
	     tagged Valid {.wr, .d} :
	       return (wr == r) ? d : rf.sub(r);
	     tagged Invalid : return rf.sub(r);
     	    endcase
	   end
   endmethod

   method Bit#(32) rd2 (Rindx r);
      if (r == 0) return 0;
      else begin
            case (rw.wget()) matches
	     tagged Valid {.wr, .d} :
	       return (wr == r) ? d : rf.sub(r);
	     tagged Invalid : return rf.sub(r);
     	    endcase
	   end
   endmethod
   
endmodule
